@@ -3,7 +3,7 @@ package Module::Lazy;
 use 5.008;
 use strict;
 use warnings;
-our $VERSION = '0.03_01';
+our $VERSION = '0.01';
 
 =head1 NAME
 
@@ -42,7 +42,6 @@ None.
 =cut
 
 use Carp;
-use constant DEBUG => $ENV{PERL_LAZYLOAD_DEBUG};
 
 =head2 import
 
@@ -63,62 +62,40 @@ No extra options (except from target module name) are allowed.
 
 =cut
 
-my $dont = $ENV{PERL_LAZYLOAD_DISABLE};
 my %seen;
-my $inc_stub = "pending load by ".__PACKAGE__;
-
 sub import {
     my ($class, $target, @rest) = @_;
 
-    # bare use statement is ok
-    return unless defined $target;
-
-    croak "Usage: use Module::Lazy 'Module::Name'; extra options not supported"
-        unless @rest == 0;
+    croak "Usage: use Module::Lazy 'Module::Name';"
+        unless defined $target and @rest == 0;
 
     # return ASAP if already loaded by us or Perl itself
     return if $seen{$target};
     my $mod = $target;
     $mod =~ s,::,/,g;
     $mod .= ".pm";
-
     return if $INC{$mod};
 
-    carp __PACKAGE__.": request to load $target "
-        .($seen{$target} ? '(seen)' : '(first time)')
-            if DEBUG;
-    return _load( $target, $mod )
-        if $dont;
-
     croak "Bad module name '$target'"
         unless $target =~ /^[A-Za-z_][A-Za-z_0-9]*(?:::[A-Za-z_0-9]+)*$/;
 
     $seen{$target} = $mod;
 
-    # If $target is later require'd directly,
-    # autoload and destroy will be overwritten and will cause a warning.
-    # Preventing them from being loaded seems like a lesser evil.
-    $INC{$mod} = $inc_stub;
-
-    _set_symbol( $target, AUTOLOAD => sub {
-        our $AUTOLOAD;
-        $AUTOLOAD =~ s/.*:://;
+    our $AUTOLOAD;
+    _set_function( $target, AUTOLOAD => sub {
+        _load( $target );
         my $jump = _jump( $target, $AUTOLOAD );
         goto $jump;
     } );
 
-    # Provide DESTROY just in case someone blesses an object directly
-    #     without ever loading a module
-    _set_symbol( $target, DESTROY => _jump( $target, DESTROY => "no_die" ) );
-
-    # If somebody calls Module->can("foo"), we can't really tell
-    # without loading, so override it
     foreach (qw( can isa )) {
-        _set_symbol( $target, $_ => _jump( $target, $_ ) );
+        my $name = $_; # separate variable to close over
+        _set_function( $target, $name => sub {
+            _load( $target );
+            my $jump = _jump( $target, $name );
+            goto $jump;
+        });
     };
-
-    # Provide a fake version for `use My::Module 100.500`
-    _set_symbol( $target, VERSION => 10**9 );
 };
 
 =head2 unimport
@@ -140,145 +117,76 @@ sub unimport {
     croak "usage: no Module::Lazy;"
         if @_;
 
-    carp __PACKAGE__.": unimport called"
-        if DEBUG;
-
-    $dont++;
     # sort keys to ensure load order stability in case of bugs
     foreach (sort keys %seen) {
-        # some modules may have been already loaded, skip if so
-        _inflate($_) if $seen{$_};
+        _load($_);
     };
 };
 
-my %cleanup_symbol;
-sub _inflate {
+my %known_method;
+sub _load {
     my $target = shift;
 
-    # TODO distinguish between "not seen" and "already loaded"
     my $mod = delete $seen{$target};
-    croak "Module '$target' is unknown Module::Lazy, or already loaded. Please file a bug"
+    croak "Module '$target' was never loaded via Module::Lazy, that's possibly a bug"
         unless $mod;
 
-    carp "Module '$target' wasn't preloaded by Module::Lazy. Please file a bug"
-        unless $INC{$mod};
-
-    return carp "Module '$target' already loaded elsewhere from '$INC{$mod}'"
-        unless $INC{$mod} eq $inc_stub;
-
     # reset stub methods prior to loading
-    foreach (keys %{ $cleanup_symbol{$target} || {} }) {
-        _unset_symbol( $target, $_ );
+    foreach (keys %{ $known_method{$target} || {} }) {
+        _set_function( $target, $_ => undef );
     };
 
-    # reset fake $VERSION
-    _set_symbol( $target, VERSION => undef );
-
-    # make the module loadable again
-    delete $INC{$mod};
-    _load( $target, $mod );
-};
-
-sub _load {
-    my ($target, $mod) = @_;
-
-    carp __PACKAGE__.": loading $target from $mod"
-        if DEBUG;
-
     package
         Module::Lazy::_::quarantine;
 
     local $Carp::Internal{ __PACKAGE__ } = 1;
     require $mod;
-    # TODO maybe $target->import()
+    # TODO maybe import()
 };
 
 sub _jump {
     my ($target, $todo, $nodie) = @_;
 
-    return sub {
-        _inflate( $target );
+    $todo =~ s/.*:://;
+    my $jump = $target->can($todo);
 
-        my $jump = $target->can($todo);
-        goto $jump
-            if $jump; # TODO should also check it's a CODEREF
+    croak qq{Can't locate object method "$todo" via package "$target"}
+        unless $jump or $nodie;
 
-        croak qq{Can't locate object method "$todo" via package "$target"}
-            unless $nodie;
-    };
+    return $jump;
 };
 
-sub _set_symbol {
-    my ($target, $name, $ref) = @_;
+sub _set_function {
+    my ($target, $name, $code) = @_;
 
-    if (ref $ref) {
-        # really update symbol table
-        $cleanup_symbol{$target}{$name}++;
+    if (ref $code) {
+        $known_method{$target}{$name}++;
         no strict 'refs'; ## no critic
-        *{ $target."::".$name } = $ref;
+        *{ $target."::".$name } = $code;
     } else {
-        # just set scalar
         no strict 'refs'; ## no critic
-        ${ $target.'::'.$name } = $ref;
+        delete ${ $target."::" }{ $name };
     };
 };
 
-sub _unset_symbol {
-    my ($target, $name) = @_;
-
-    no strict 'refs'; ## no critic
-    # because package scalars are _special_,
-    # move SCALAR ref around the destruction
-    # just in case someone referenced it before module was loaded
-    my $save = \${ $target."::".$name };
-    delete ${ $target."::" }{ $name };
-    *{ $target.'::'.$name } = $save;
-};
-
-=head1 ENVIRONMENT
-
-If C<PERL_LAZYLOAD_DEBUG> is set and true,
-warns about module loading via Carp.
-
-If C<PERL_LAZYLOAD_DISABLE> is set and true,
-don't try to lazyload anything - just go straight to C<require>.
-
-(That's roughly equivalent to C<perl -M-Module::Lazy> on command line).
-
-=head1 CAVEATS
-
-=over
-
-=item * The following symbols are currently replaced by stubs
-in the module to be loaded: C<AUTOLOAD>, C<DESTROY>, C<can>, C<isa>.
-
-=item * If a module was ever lazyloaded, a normal C<require> would do nothing.
-A method must be called to inflate the module.
-
-This is done so because a normal require would partially overwrite
-stub methods and potentially wreak havoc.
-
-=item * A fake $VERSION = 10**9 is generated so that C<use Module x.yy>
-doesn't die. This value is erased before actually loading the module.
-
-=back
-
+=head1 AUTHOR
 
+Konstantin S. Uvarin, C<< <khedin@cpan.org> >>
 
 =head1 BUGS
 
 =over
 
-=item * C<use mro 'c3';> does not work with lazy-loaded parent classes.
-
-=item * C<import()> is not called on the modules being loaded.
+=item * import() is not called on the modules being loaded.
 The decision is yet to be made whether it's good or bad.
 
+=item * C<no Module::Lazy> should prevent further demand-loading.
+
 =item * no way to preload prototyped exported functions
 (that's what L<autouse> does),
 but maybe there should be?
 
-=item * certainly not enough interoperability tests.
+=item * certainly not enough interoperability tests (C<use mro 'c3'>?).
 
 =back
 
@@ -333,7 +241,7 @@ it does it for imported functions rather than methods.
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2019 Konstantin S. Uvarin, C<< <khedin@cpan.org> >>
+Copyright 2019 Konstantin S. Uvarin.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of the the Artistic License (2.0). You may obtain a
